<?php
/**
 * @brief        Yellow Pages Application Class
 * @author        <a href=''>Invisionify.com</a>
 * @copyright    (c) 2020 Invisionify.com
 * @package        Invision Community
 * @subpackage    Yellow Pages
 * @since        12 Oct 2020
 * @version
 */

namespace IPS\ypdatabase;

/**
 * Yellow Pages Application Class
 */
class _Application extends \IPS\Application
{

    /**
     * Install 'other' items.
     *
     * @return void
     */
    public function installOther()
    {
        /* import database, see \IPS\cms\modules\admin\databases\databases for reference */
        $xml = new \IPS\Xml\XMLReader;
        $xml->open(\IPS\Application::getRootPath() . "/applications/ypdatabase/interface/database.xml");

        if (!@$xml->read()) {
            \IPS\Output::i()->error('xml_upload_invalid', '2Y100/1', 403, '');
        }

        $database = new \IPS\cms\Databases;
        $database->key = 'import_' . mt_rand();
        $database->save();

        /* Set default perms, these will be editable post DB import */
        $groups = array();
        foreach (\IPS\Db::i()->select('row_id', 'core_admin_permission_rows', array('row_id_type=?', 'group')) as $row) {
            $groups[] = $row;
        }
        $default = implode(',', $groups);
        \IPS\Db::i()->insert('core_permission_index', array(
            'app' => 'cms',
            'perm_type' => 'databases',
            'perm_type_id' => $database->id,
            'perm_view' => '*', # view
            'perm_2' => '*', # read
            'perm_3' => $default, # add
            'perm_4' => $default, # edit
            'perm_5' => $default, # reply
            'perm_6' => $default  # rate
        ));

        $json = json_decode(@file_get_contents(\IPS\ROOT_PATH . "/applications/cms/data/databaseschema.json"), true);
        $table = $json['cms_custom_database_1'];

        $table['name'] = 'cms_custom_database_' . $database->id;
        // there should not be any custom fields in json schema. I am not sure why IPS checks it.
        foreach ($table['columns'] as $name => $data) {
            if (mb_substr($name, 0, 6) === 'field_') {
                unset($table['columns'][$name]);
            }
        }
        foreach ($table['indexes'] as $name => $data) {
            if (mb_substr($name, 0, 6) === 'field_') {
                unset($table['indexes'][$name]);
            }
        }

        // now let's create a database table
        try {
            if (!\IPS\Db::i()->checkForTable($table['name'])) {
                \IPS\Db::i()->createTable($table);
            }
        } catch (\IPS\Db\Exception $ex) {
            throw new \LogicException($ex);
        }

        /* now define language strings */
        $langs = array();
        foreach (array('content_db_lang_sl', 'content_db_lang_pl', 'content_db_lang_su', 'content_db_lang_pu', 'content_db_lang_ia') as $lang) {
            if ($xml->getAttribute($lang)) {
                $langs[$lang] = $xml->getAttribute($lang);
                \IPS\Lang::saveCustom('cms', $lang . "_" . $database->id, $xml->getAttribute($lang));
                if ($lang === 'content_db_lang_pu') {
                    \IPS\Lang::saveCustom('cms', $lang . "_" . $database->id . '_pl', $xml->getAttribute($lang));
                }
            }
        }

        /* Other data */
        while ($xml->read()) {
            $name = NULL;
            $desc = NULL;

            if ($xml->name == 'data') {
                /* Here we can set anything we like to be saved in cms_databases, those are attributes on data element  */
                $node = new \SimpleXMLElement($xml->readOuterXML());
                foreach ($node->attributes() as $k => $v) {
                    $database->$k = (string)$v;
                }

                /* Lets see if there are some children that cannot be saved one to one into cms_databases */
                foreach ($node->children() as $k => $v) {
                    if ($k === 'name') {
                        $name = $v;
                    } else if ($k === 'description') {
                        $desc = $v;
                    } else {
                        $tryJson = json_decode($v, TRUE);
                        $database->$k = ($tryJson) ? $tryJson : (string)$v;
                    }
                }

                /* And save some custom language strings for the database */
                \IPS\Lang::saveCustom('cms', "content_db_" . $database->id, $name);
                \IPS\Lang::saveCustom('cms', "content_db_" . $database->id . '_desc', $desc);
                \IPS\Lang::saveCustom('cms', "digest_area_cms_records" . $database->id, $langs['content_db_lang_pu']);
                \IPS\Lang::saveCustom('cms', "digest_area_cms_categories" . $database->id, $langs['content_db_lang_pu']);

                $menu = array();

                foreach (\IPS\Lang::languages() as $id => $lang) {
                    $menu[$lang->_id] = \sprintf($lang->get('cms_create_menu_records_x'), $langs['content_db_lang_su'], $name);
                }

                /* Notification, search/followed/new content langs */
                \IPS\Lang::saveCustom('cms', "cms_create_menu_records_" . $database->id, $menu);
                \IPS\Lang::saveCustom('cms', "cms_records" . $database->id . '_pl', $langs['content_db_lang_su']);
                \IPS\Lang::saveCustom('cms', "module__cms_records" . $database->id, $name);

                /* We stop reading XML file at this point */
                break;
            }
        }

        // we have to set this manually due to the bug https://invisioncommunity.com/forums/topic/459055-bug-thumbs-settings-for-database-is-not-imported/
        $database->fixed_field_settings = json_encode(array(
            'record_image' => array(
                'image_dims' => array(0, 0),
                'thumb_dims' => array(200, 200),
            )
        ));
        $database->save();

        /* set up some stores (and make sure they are clean from previous possibly broken imports */
        \IPS\Data\Store::i()->db_import_cat_map = array();
        \IPS\Data\Store::i()->db_import_cat_parent_map = array();
        \IPS\Data\Store::i()->db_import_field_map = array();
        \IPS\Data\Store::i()->db_import_db_id = $database->id;
        \IPS\Settings::i()->changeValues(array('yp_database_id' => $database->id));

        $this->_import_database($xml, $database);
        $this->_create_record($database, $default);

        /* Create the page */
        $page_id = 0;
        try {
            $page_id = \IPS\Db::i()->select('page_id', 'cms_pages', array('page_seo_name=?', 'yellowpages'))->first();
        } catch (\UnderflowException $e) {
        }
        $pageValues = array(
            'page_name' => "Yellow Pages",
            'page_title' => "Yellow Pages",
            'page_seo_name' => ($page_id) ? 'yellowpages' . mt_rand() : 'yellowpages',
            'page_folder_id' => 0,
            'page_ipb_wrapper' => 1,
            'page_show_sidebar' => 1,
            'page_type' => 'builder',
            'page_template' => 'page_builder__single_column__page_page_builder_single_column',
        );

        try {
            $page = \IPS\cms\Pages\Page::createFromForm($pageValues, 'html');
        } catch (\Exception $ex) {
            \IPS\Log::log($ex, 'pages_create_page_error');
        }
        \IPS\Db::i()->replace('core_permission_index', array(
            'app' => 'cms',
            'perm_type' => 'pages',
            'perm_type_id' => $page->id,
            'perm_view' => '*'
        ));

        \IPS\Settings::i()->changeValues(array('yp_page_id' => $page->id));
        $database->page_id = $page->id;
        $database->save();

        unset(\IPS\Data\Store::i()->pages_page_urls);

        $defaultNavigation = $this->defaultFrontNavigation();
        foreach ($defaultNavigation as $type => $tabs) {
            foreach ($tabs as $config) {
                $config['real_app'] = 'cms';
                $config['app'] = 'cms';

                \IPS\core\FrontNavigation::insertMenuItem(NULL, $config, \IPS\Db::i()->select('MAX(position)', 'core_menu')->first());
            }
        }
        unset(\IPS\Data\Store::i()->frontNavigation);

        /* Default database widget */
        $col1HorizontalWidgets[] = array(
            'app' => 'cms',
            'key' => 'Database',
            'unique' => mt_rand(),
            'configuration' => array('database' => $database->id)
        );

        \IPS\Db::i()->insert('cms_page_widget_areas', array(
            'area_page_id' => $page->id,
            'area_widgets' => json_encode($col1HorizontalWidgets),
            'area_area' => 'col1',
            'area_orientation' => 'horizontal'
        ));

        /* Default category widget */
        $sidebarVerticalWidgets[] = array(
            'app' => 'cms',
            'key' => 'Categories',
            'unique' => mt_rand(),
            'configuration' => array()
        );

        /* Default category widget */
        $sidebarVerticalWidgets[] = array(
            'app' => 'ypdatabase',
            'key' => 'ypLocationsBlock',
            'unique' => mt_rand(),
            'configuration' => array()
        );

        \IPS\Db::i()->insert('cms_page_widget_areas', array(
            'area_page_id' => $page->id,
            'area_widgets' => json_encode($sidebarVerticalWidgets),
            'area_area' => 'sidebar',
            'area_orientation' => 'vertical'
        ));

        $this->_create_blocks();
    }

    private function _import_database($xml, $database)
    {
        /* Now lets import categories and fields */
        $data = array('next' => 'field', 'done' => 0);
        $i = 0;

        $xml->read();
        $next = NULL;
        /* We do handle templates via app to make them upgradable */
        $areas = array('field', 'category');

        if ($data['next']) {
            while ($xml->read()) {
                /* We do skip anything that is not field or category */
                if (!\in_array($xml->name, $areas) or $xml->nodeType != \XMLReader::ELEMENT) {
                    continue;
                }

                $i++;
                if ($data['done']) {
                    if ($i - 1 < $data['done']) {
                        $xml->next();
                        continue;
                    }
                }

                $doneSomething = false;
                if ($xml->name == $data['next']) {
                    $data['done']++;

                    $areaName = $xml->name;
                    $node = new \SimpleXMLElement($xml->readOuterXML());

                    /* Import */
                    switch ($areaName) {
                        case 'field':
                            $attrs = array();
                            $perms = array();
                            /* We are able to fill the table cms_database_fields via XML attributes */
                            foreach ($node->attributes() as $k => $v) {
                                if (mb_substr($k, 0, 5) === 'perm_' or \in_array($k, array('app', 'owner_only', 'friend_only', 'authorized_users'))) {
                                    $perms[$k] = (string)$v;
                                } else {
                                    $attrs[$k] = (string)$v;
                                }
                            }

                            /* XML children store data that cannot be written in cms_database_fields directly */
                            $displayJson = NULL;
                            foreach ($node->children() as $k => $v) {
                                if ($k == 'field_name') {
                                    $k = 'field_title';
                                }

                                $tryJson = json_decode($v, TRUE);

                                if ($k === 'field_display_json' and \is_array($tryJson)) {
                                    $displayJson = $tryJson;
                                    continue;
                                }

                                $attrs[$k] = ($tryJson) ? $tryJson : (string)$v;
                            }

                            /* Check to ensure we have the same field type, if not default to text so data is retained (buildHelper also does this */
                            if (!class_exists('\IPS\cms\Fields\\' . mb_ucfirst($attrs['field_type'])) and !class_exists('\IPS\Helpers\Form\\' . mb_ucfirst($attrs['field_type']))) {
                                $attrs['field_type'] = 'Text';
                            }

                            $attrs['_skip_formatting'] = TRUE;

                            $originalFieldId = $attrs['field_id'];
                            unset($attrs['field_id']);

                            $fieldsClass = 'IPS\cms\Fields' . $database->id;

                            $obj = new $fieldsClass;
                            $values = $obj->formatFormValues($attrs);

                            if (isset($attrs['field_extra'])) {
                                /* The export format is always correct */
                                $values['field_extra'] = $attrs['field_extra'];
                            }

                            // Create a new field in cms_database_fields
                            $obj->saveForm($values);

                            if ($displayJson) {
                                $obj->display_json = $displayJson;
                                $obj->save();
                            }

                            if ($originalFieldId == $database->field_title) {
                                $database->field_title = $obj->id;
                                $database->save();
                            } else if ($originalFieldId == $database->field_content) {
                                $database->field_content = $obj->id;
                                $database->save();
                            }

                            $map = \IPS\Data\Store::i()->db_import_field_map;

                            $map[$originalFieldId] = $obj->id;

                            \IPS\Data\Store::i()->db_import_field_map = $map;

                            $existingPerms = $obj->permissions();
                            $newPerms = array_merge($perms, array('perm_id' => $existingPerms['perm_id'], 'perm_type_id' => $obj->id));

                            /* Set default permissions if not defined in export */
                            if (!isset($perms['perm_view'])) {
                                $newPerms['perm_view'] = '*';
                            }

                            if (!isset($perms['perm_2'])) {
                                $newPerms['perm_2'] = \IPS\Settings::i()->admin_group;
                            }

                            if (!isset($perms['perm_3'])) {
                                $newPerms['perm_3'] = \IPS\Settings::i()->admin_group;
                            }

                            \IPS\Db::i()->update('core_permission_index', $newPerms, array('perm_id=?', $existingPerms['perm_id']));

                            \IPS\Lang::saveCustom('cms', "content_field_" . $obj->id, $attrs['field_title']);

                            if (isset($attrs['field_description'])) {
                                \IPS\Lang::saveCustom('cms', "content_field_" . $obj->id . '_desc', $attrs['field_description']);
                            }

                            if (isset($attrs['field_validator_error'])) {
                                \IPS\Lang::saveCustom('cms', "content_field_" . $obj->id . '_validation_error', $attrs['field_validator_error']);
                            }
                            $next = 'category';
                            $doneSomething = true;
                            break;

                        case 'category':
                            $attrs = array();
                            $perms = array();
                            /* Again we are able to fill the table cms_database_categories via XML attributes */
                            foreach ($node->attributes() as $k => $v) {
                                if (mb_substr($k, 0, 5) === 'perm_' or \in_array($k, array('app', 'owner_only', 'friend_only', 'authorized_users'))) {
                                    $perms[$k] = (string)$v;
                                } else {
                                    $attrs[$k] = (\in_array($k, array('category_parent_id'))) ? (int)$v : (string)$v;
                                }
                            }

                            /* Any kids of your own? */
                            foreach ($node->children() as $k => $v) {
                                $tryJson = json_decode($v, TRUE);
                                $attrs[$k] = ($tryJson) ? $tryJson : (string)$v;
                            }

                            $originalCategoryId = $attrs['category_id'];
                            $originalParentId = $attrs['category_parent_id'];

                            unset($attrs['category_id']);
                            $attrs['category_parent_id'] = 0;

                            /* Create a category */
                            $category = new \IPS\cms\Categories;
                            $category->database_id = $database->id;
                            $category::$permType = 'categories_' . $database->id;

                            $category->saveForm($category->formatFormValues($attrs));

                            if ($category->fields and $category->fields !== '*') {
                                if (\is_array($category->fields)) {
                                    $field_map = \IPS\Data\Store::i()->db_import_field_map;
                                    $newMap = array();

                                    foreach ($category->fields as $fid) {
                                        if (isset($field_map[$fid])) {
                                            $newMap[] = $field_map[$fid];
                                        }
                                    }

                                    if (\count($newMap)) {
                                        $category->fields = json_encode($newMap);
                                        $category->save();
                                    }
                                }
                            }

                            $cat_map = \IPS\Data\Store::i()->db_import_cat_map;
                            $parent_map = \IPS\Data\Store::i()->db_import_cat_parent_map;

                            $cat_map[$originalCategoryId] = $category->id;
                            $parent_map[$category->id] = $originalParentId;

                            \IPS\Data\Store::i()->db_import_cat_map = $cat_map;
                            \IPS\Data\Store::i()->db_import_cat_parent_map = $parent_map;

                            /* Perms */
                            $existingPerms = $category->permissions();

                            \IPS\Db::i()->update('core_permission_index', array_merge($perms, array('perm_id' => $existingPerms['perm_id'], 'perm_type_id' => $category->id)), array('perm_id=?', $existingPerms['perm_id']));

                            $doneSomething = true;
                            $next = NULL; /* Templates will be handles via app to make them upgradable */
                            break;

                    }
                } else {
                    /* Did we do anything? if not, skip to the next section */
                    if (!$doneSomething) {
                        switch ($data['next']) {
                            case 'field':
                                $data['next'] = 'category';
                                break;
                            case 'category':
                                $data['next'] = NULL;
                                break;
                        }
                    }

                    $xml->next();
                }
            }
            /* Done */
            $data['next'] = $next;
        }

        /* Remap categories */
        if (isset(\IPS\Data\Store::i()->db_import_cat_map) and isset(\IPS\Data\Store::i()->db_import_cat_parent_map)) {
            foreach (\IPS\Data\Store::i()->db_import_cat_parent_map as $new => $oldParent) {
                if ($oldParent > 0) {
                    if (isset(\IPS\Data\Store::i()->db_import_cat_map[$oldParent])) {
                        $cat = \IPS\cms\Categories::load($new);
                        $cat->parent_id = \IPS\Data\Store::i()->db_import_cat_map[$oldParent];
                        $cat->save();
                    }
                }
            }
        }

        /* SORT IT OUT! */
        if (isset(\IPS\Data\Store::i()->db_import_field_map)) {
            $map = \IPS\Data\Store::i()->db_import_field_map;
            $sortId = \intval(mb_substr($database->field_sort, 6));

            if (isset($map[$sortId])) {
                $database->field_sort = 'field_' . $map[$sortId];
                $database->save();
            }
        }
    }

    private function _create_record($database, $default)
    {
        /* Create default record */
        $item = 'IPS\cms\Records' . $database->id;
        $comment = 'IPS\cms\Records\Comment' . $database->id;
        $container = 'IPS\cms\Categories' . $database->id;

        $content = <<<EOF
         <p>Your first company is already here!</p>
         <p>This is a sample record that can be deleted or customized. This is a place where the owner of the company (the member who created it) can edit a company description. You can change the owner in your AdminCP.</p>
         <p>Create new categories and companies to start to fill your business directory. Check the permissions of WHO can add in your group permissions and if the companies must be approved before getting visible to the members.</p>
         <p><a href="https://invisionify.com">Get Support for the app and database</a></p>
EOF;
        $titleField = 'field_' . $database->field_title;
        $contentField = 'field_' . $database->field_content;
        $category = $container::load($database->_default_category);
        $member = \IPS\Member::loggedIn()->member_id ? \IPS\Member::loggedIn() : \IPS\Member::load(1);
        $record = $item::createItem($member, \IPS\Request::i()->ipAddress(), \IPS\DateTime::ts(time()), $category, FALSE);
        $record->$titleField = "One company";
        $record->$contentField = $content;
        $record_image = \IPS\File::create('cms_Records', 'record_image.jpg', file_get_contents(\IPS\Application::getRootPath() . '/applications/ypdatabase/interface/yprimage.jpg'));
        $record->record_image = $record_image->url;
        $record->record_image_thumb = (string)$record_image->thumbnail('cms_Records', 200, 200);
        $record->record_featured = 1;

        $customFields = \IPS\Db::i()->select('field_id, field_key', 'cms_database_fields', array('field_database_id=?', \IPS\Settings::i()->yp_database_id));
        $yp_fields = array();
        foreach ($customFields as $field) {
            $yp_fields[$field['field_key']] = 'field_' . $field['field_id'];
        }

        // these lines are here to bypass a bug https://invisioncommunity.com/forums/topic/458936-multiple-upload-field-while-importing-database/
        $fieldsClass = 'IPS\cms\Fields' . $database->id;
        $gallery = $fieldsClass::load('yp_gallery', 'field_key');
        $gallery->is_multiple = 1;
        $gallery->extra = json_encode(array(
                'type' => 'image',
                'maxsize' => array(0, 0),
                'thumbsize' => array(200, 200),
            )
        );
        $gallery->save();
        // fix permissions
        \IPS\Db::i()->replace('core_permission_index', array(
            'app' => 'cms',
            'perm_type' => 'fields',
            'perm_type_id' => $gallery->id,
            'perm_view' => '*',
            'perm_2' => '*', # read
            'perm_3' => $default, # add
        ));

        $address = new \IPS\GeoLocation;
        $address->addressLines = array('Invision Power Services, Inc.', 'PO Box 2365');
        $address->city = 'Forest';
        $address->region = 'VA';
        $address->country = 'US';
        $address->postalCode = '24551';
        $record->{$yp_fields['yp_address']} = json_encode($address);
        $record->{$yp_fields['yp_hours']} = 'Monday - Thursday: 08:00 - 17:00,
        Friday & Saturday: 08:00 - 14:00';
        $record->{$yp_fields['yp_website']} = 'https://invisionify.com';
        $record->{$yp_fields['yp_phone']} = '1234567';
        $record->{$yp_fields['yp_email']} = 'support@invisionify.com';
        $record->{$yp_fields['yp_facebook']} = 'https://www.facebook.com/invisioncommunityofficial/';
        $record->{$yp_fields['yp_twitter']} = 'https://twitter.com/invisionps';
        $record->{$yp_fields['yp_youtube']} = 'https://www.youtube.com/user/IPSTutorials';
        $record->{$yp_fields['yp_linkedin']} = 'https://www.linkedin.com/company/invision-power-services';
        $record->{$yp_fields['yp_instagram']} = 'https://www.instagram.com/cats_of_instagram/';
        $record->save();

        /* create gallery examples */
        $gallery_images = array();
        for ($i = 1; $i <= 7; $i++) {
            $gallery_image = \IPS\File::create('cms_Records', 'ypgal' . $i . '.jpg', file_get_contents(\IPS\Application::getRootPath() . '/applications/ypdatabase/interface/ypgal' . $i . '.jpg'));
            $gallery_thumb = $gallery_image->thumbnail('cms_Records', 200, 200, true);
            \IPS\Db::i()->insert('cms_database_fields_thumbnails', array(
                'thumb_original_location' => (string)$gallery_image,
                'thumb_location' => (string)$gallery_thumb,
                'thumb_field_id' => $gallery->id,
                'thumb_database_id' => $database->id,
                'thumb_record_id' => $record->_id
            ));
            $gallery_images[] = (string)$gallery_image;
        }
        $record->{$yp_fields['yp_gallery']} = implode(',', $gallery_images);
        $record->save();

        \IPS\Content\Search\Index::i()->index($record);

        $category->last_record_date = time();
        $category->save();
    }

    private function _create_blocks()
    {
        $container_id = 0;
        try {
            $container_id = \IPS\Db::i()->select('container_id', 'cms_containers', array('container_key=?', 'block_yp'))->first();
        } catch (\UnderflowException $e) {
        }
        $container = new \IPS\cms\Blocks\Container;
        $container->parent_id = 0;
        $container->name = ($container_id) ? 'Yellow Pages' . mt_rand() : 'Yellow Pages';
        $container->type = 'block';
        $container->key = 'block_yp';
        $container->save();

        /* get template keys by title */
        $templates = iterator_to_array(\IPS\DB::i()->select('template_key, template_title', 'cms_templates', array("template_title IN ('yellowpages_feed', 'yellowpages_featured')")));
        $block_templates = array();
        foreach ($templates as $template) {
            $block_templates[$template['template_title']] = $template['template_key'];
        }

        /* yp_header_block */
        try {
            $block_key = \IPS\Db::i()->select('block_id', 'cms_blocks', array('block_key=?', 'yp_header_block'))->first();
            $block = \IPS\cms\Blocks\Block::load($block_key);
            $block->delete();
        } catch (\UnderflowException $e) {
        }
        $block = new \IPS\cms\Blocks\Block;
        $block->category = $container->id;
        $block->type = "custom";
        $block->key = "yp_header_block";
        $block->content = <<<EOF
        {{if \IPS\Settings::i()->yp_enable_header == 1}}
        {{\$widget = \IPS\Widget::load( \IPS\Application::load( 'ypdatabase' ), 'ypHeaderBlock', mt_rand(), [], NULL, 'horizontal' );}}
        {\$widget|raw}
        {{endif}}
EOF;
        $block->active = 1;
        $block->save();
        \IPS\Lang::saveCustom('cms', "content_block_name_" . $block->id, 'Header Block');
        \IPS\Lang::saveCustom('cms', "content_block_name_" . $block->id . '_desc', 'Displays stylish header for Yellow Pages');
        \IPS\Db::i()->insert('core_permission_index', array(
            'app' => 'cms',
            'perm_type' => 'blocks',
            'perm_type_id' => $block->id,
            'perm_view' => '*', # view
        ));

        /* let's set background for header block */
        $image = \IPS\File::create('ypdatabase_Blocks', 'ypheader.jpg', file_get_contents(\IPS\Application::getRootPath() . '/applications/ypdatabase/interface/ypheader.jpg'));
        \IPS\Settings::i()->changeValues(array('yp_header_image' => $image->url));

        /* yp_top_rated */
        try {
            $block_key = \IPS\Db::i()->select('block_id', 'cms_blocks', array('block_key=?', 'yp_top_rated'))->first();
            $block = \IPS\cms\Blocks\Block::load($block_key);
            $block->delete();
        } catch (\UnderflowException $e) {
        }
        $block = new \IPS\cms\Blocks\Block;
        $block->category = $container->id;
        $block->type = "plugin";
        $block->plugin = "RecordFeed";
        $block->plugin_app = "cms";
        $block->key = "yp_top_rated";
        $block->content = NULL;
        $config = array();
        $config['cms_rf_database'] = \IPS\Settings::i()->yp_database_id;
        $config['widget_feed_sort_on'] = 'record_rating';
        $config['widget_feed_sort_dir'] = 'desc';
        $config['widget_feed_show'] = 10;
        /* Copy language bits to make them unique */
        $config['language_key'] = 'widget_title_' . md5(mt_rand());
        $block->plugin_config = json_encode($config);
        $block->template = $block_templates['yellowpages_feed'];
        $block->active = 1;
        $block->save();
        \IPS\Lang::saveCustom('cms', "content_block_name_" . $block->id, 'Top Rated');
        \IPS\Lang::saveCustom('cms', "content_block_name_" . $block->id . '_desc', 'Displays top rated records');
        \IPS\Lang::saveCustom('core', $config['language_key'], 'Top Rated');
        \IPS\Db::i()->insert('core_permission_index', array(
            'app' => 'cms',
            'perm_type' => 'blocks',
            'perm_type_id' => $block->id,
            'perm_view' => '*', # view
        ));

        /* yp_recently_added */
        try {
            $block_key = \IPS\Db::i()->select('block_id', 'cms_blocks', array('block_key=?', 'yp_recently_added'))->first();
            $block = \IPS\cms\Blocks\Block::load($block_key);
            $block->delete();
        } catch (\UnderflowException $e) {
        }
        $block = new \IPS\cms\Blocks\Block;
        $block->category = $container->id;
        $block->type = "plugin";
        $block->plugin = "RecordFeed";
        $block->plugin_app = "cms";
        $block->key = "yp_recently_added";
        $block->content = NULL;
        $config = array();
        $config['cms_rf_database'] = \IPS\Settings::i()->yp_database_id;
        $config['widget_feed_sort_on'] = 'primary_id_field';
        $config['widget_feed_sort_dir'] = 'desc';
        $config['widget_feed_show'] = 10;
        /* Copy language bits to make them unique */
        $config['language_key'] = 'widget_title_' . md5(mt_rand());
        $block->plugin_config = json_encode($config);
        $block->template = $block_templates['yellowpages_feed'];
        $block->active = 1;
        $block->save();
        \IPS\Lang::saveCustom('cms', "content_block_name_" . $block->id, 'Recently Added');
        \IPS\Lang::saveCustom('cms', "content_block_name_" . $block->id . '_desc', 'Displays recently added records');
        \IPS\Lang::saveCustom('core', $config['language_key'], 'Recently Added');
        \IPS\Db::i()->insert('core_permission_index', array(
            'app' => 'cms',
            'perm_type' => 'blocks',
            'perm_type_id' => $block->id,
            'perm_view' => '*', # view
        ));

        /* yp_featured */
        try {
            $block_key = \IPS\Db::i()->select('block_id', 'cms_blocks', array('block_key=?', 'yp_featured'))->first();
            $block = \IPS\cms\Blocks\Block::load($block_key);
            $block->delete();
        } catch (\UnderflowException $e) {
        }
        $block = new \IPS\cms\Blocks\Block;
        $block->category = $container->id;
        $block->type = "plugin";
        $block->plugin = "RecordFeed";
        $block->plugin_app = "cms";
        $block->key = "yp_featured";
        $block->content = NULL;
        $config = array();
        $config['cms_rf_database'] = \IPS\Settings::i()->yp_database_id;
        $config['widget_feed_sort_on'] = 'record_publish_date';
        $config['widget_feed_sort_dir'] = 'desc';
        $config['widget_feed_status_featured'] = 'featured';
        $config['widget_feed_show'] = 3;
        /* Copy language bits to make them unique */
        $config['language_key'] = 'widget_title_' . md5(mt_rand());
        $block->plugin_config = json_encode($config);
        $block->template = $block_templates['yellowpages_featured'];
        $block->active = 1;
        $block->save();
        \IPS\Lang::saveCustom('cms', "content_block_name_" . $block->id, 'Featured Listings');
        \IPS\Lang::saveCustom('cms', "content_block_name_" . $block->id . '_desc', 'Displays featured records');
        \IPS\Lang::saveCustom('core', $config['language_key'], 'Featured Listings');
        \IPS\Db::i()->insert('core_permission_index', array(
            'app' => 'cms',
            'perm_type' => 'blocks',
            'perm_type_id' => $block->id,
            'perm_view' => '*', # view
        ));

        /* yp_latest_updated */
        try {
            $block_key = \IPS\Db::i()->select('block_id', 'cms_blocks', array('block_key=?', 'yp_latest_updated'))->first();
            $block = \IPS\cms\Blocks\Block::load($block_key);
            $block->delete();
        } catch (\UnderflowException $e) {
        }
        $block = new \IPS\cms\Blocks\Block;
        $block->category = $container->id;
        $block->type = "plugin";
        $block->plugin = "RecordFeed";
        $block->plugin_app = "cms";
        $block->key = "yp_latest_updated";
        $block->content = NULL;
        $config = array();
        $config['cms_rf_database'] = \IPS\Settings::i()->yp_database_id;
        $config['widget_feed_sort_on'] = 'record_updated';
        $config['widget_feed_sort_dir'] = 'desc';
        $config['widget_feed_show'] = 10;
        /* Copy language bits to make them unique */
        $config['language_key'] = 'widget_title_' . md5(mt_rand());
        $block->plugin_config = json_encode($config);
        $block->template = $block_templates['yellowpages_feed'];
        $block->active = 1;
        $block->save();
        \IPS\Lang::saveCustom('cms', "content_block_name_" . $block->id, 'Latest Updated');
        \IPS\Lang::saveCustom('cms', "content_block_name_" . $block->id . '_desc', 'Displays newly updated records');
        \IPS\Lang::saveCustom('core', $config['language_key'], 'Latest Updates');
        \IPS\Db::i()->insert('core_permission_index', array(
            'app' => 'cms',
            'perm_type' => 'blocks',
            'perm_type_id' => $block->id,
            'perm_view' => '*', # view
        ));

        /* yp_most_viewed */
        try {
            $block_key = \IPS\Db::i()->select('block_id', 'cms_blocks', array('block_key=?', 'yp_most_viewed'))->first();
            $block = \IPS\cms\Blocks\Block::load($block_key);
            $block->delete();
        } catch (\UnderflowException $e) {
        }
        $block = new \IPS\cms\Blocks\Block;
        $block->category = $container->id;
        $block->type = "plugin";
        $block->plugin = "RecordFeed";
        $block->plugin_app = "cms";
        $block->key = "yp_most_viewed";
        $block->content = NULL;
        $config = array();
        $config['cms_rf_database'] = \IPS\Settings::i()->yp_database_id;
        $config['widget_feed_sort_on'] = 'record_views';
        $config['widget_feed_sort_dir'] = 'desc';
        $config['widget_feed_show'] = 10;
        /* Copy language bits to make them unique */
        $config['language_key'] = 'widget_title_' . md5(mt_rand());
        $block->plugin_config = json_encode($config);
        $block->template = $block_templates['yellowpages_feed'];
        $block->active = 1;
        $block->save();
        \IPS\Lang::saveCustom('cms', "content_block_name_" . $block->id, 'Most Views');
        \IPS\Lang::saveCustom('cms', "content_block_name_" . $block->id . '_desc', 'Displays mostly viewied records');
        \IPS\Lang::saveCustom('core', $config['language_key'], 'Most Views');
        \IPS\Db::i()->insert('core_permission_index', array(
            'app' => 'cms',
            'perm_type' => 'blocks',
            'perm_type_id' => $block->id,
            'perm_view' => '*', # view
        ));
    }

    /**
     * Default front navigation
     *
     * @code
     *
     * // Each item...
     * array(
     * 'key'        => 'Example',        // The extension key
     * 'app'        => 'core',            // [Optional] The extension application. If ommitted, uses this application
     * 'config'    => array(...),        // [Optional] The configuration for the menu item
     * 'title'        => 'SomeLangKey',    // [Optional] If provided, the value of this language key will be copied to menu_item_X
     * 'children'    => array(...),        // [Optional] Array of child menu items for this item. Each has the same format.
     * )
     *
     * return array(
     * 'rootTabs'        => array(), // These go in the top row
     * 'browseTabs'    => array(),    // These go under the Browse tab on a new install or when restoring the default configuraiton; or in the top row if installing the app later (when the Browse tab may not exist)
     * 'browseTabsEnd'    => array(),    // These go under the Browse tab after all other items on a new install or when restoring the default configuraiton; or in the top row if installing the app later (when the Browse tab may not exist)
     * 'activityTabs'    => array(),    // These go under the Activity tab on a new install or when restoring the default configuraiton; or in the top row if installing the app later (when the Activity tab may not exist)
     * )
     * @endcode
     * @return array
     */
    public function defaultFrontNavigation()
    {
        $browseTabs = array();

        try {
            $page = \IPS\cms\Pages\Page::load(\IPS\Settings::i()->yp_page_id);
            $browseTabs[] = array('key' => 'Pages', 'app' => 'cms', 'config' => array('menu_content_page' => $page->id, 'menu_title_page_type' => 0));
        } catch (\OutOfRangeException $ex) {
        }

        return array(
            'rootTabs' => array(),
            'browseTabs' => $browseTabs,
            'browseTabsEnd' => array(),
            'activityTabs' => array()
        );
    }

    /**
     * [Node] Get Icon for tree
     *
     * @note	Return the class for the icon (e.g. 'globe')
     * @return	string|null
     */
    protected function get__icon()
    {
        return 'map-signs';
    }
}